@@ -31,6 +31,7 @@
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
 #include <dbus/dbus-glib.h>
+#include <libosso-abook/osso-abook.h>
 
 #include <location/location-gps-device.h>
 #include <location/location-gpsd-control.h>
@@ -47,6 +48,7 @@ static void on_locate_me (GtkButton *button, MapBuddyWindow *self);
 static void on_layer (GtkButton *button, MapBuddyWindow *self);
 static gboolean on_key_pressed (GtkWindow* window, GdkEventKey *event, MapBuddyWindow *self);
 static void on_state_changed (GObject *gobject, GParamSpec *pspec, MapBuddyWindow *self);
+static void on_zoom_level_changed (GObject *gobject, GParamSpec *pspec, MapBuddyWindow *self);
 
 static void on_location_error (LocationGPSDControl *control, LocationGPSDControlError error, MapBuddyWindow *self);
 static void on_location_changed (LocationGPSDevice *device, MapBuddyWindow *self);
@@ -61,8 +63,8 @@ static void open_url (GtkButton* button, const gchar* url);
 
 G_DEFINE_TYPE (MapBuddyWindow, map_buddy_window, HILDON_TYPE_STACKABLE_WINDOW)
 
-#define SEARCH_PLACES "Places, city, state or country"
-#define SEARCH_PRAIZED "Merchants, restaurants and businesses"
+#define SEARCH_PLACES _("Places, city, state or country")
+#define SEARCH_PRAIZED _("Merchants, restaurants and businesses")
 
 
 static void
@@ -154,7 +156,6 @@ map_buddy_window_init (MapBuddyWindow *self)
   self->gps_device = g_object_new (LOCATION_TYPE_GPS_DEVICE, NULL);
 
   g_object_set (G_OBJECT (self->gps_control),
-      "preferred-method", LOCATION_METHOD_GNSS | LOCATION_METHOD_AGNSS,
       "preferred-interval", LOCATION_INTERVAL_30S,
       NULL);
 
@@ -187,6 +188,7 @@ create_map_area (MapBuddyWindow *self)
   self->view = gtk_champlain_embed_get_view (GTK_CHAMPLAIN_EMBED (self->embed));
 
   g_object_set (self->view,
+      "scroll-mode", CHAMPLAIN_SCROLL_MODE_KINETIC,
       "min-zoom-level", 2, /* Fill the whole screen */
 #if CHAMPLAIN_CHECK_VERSION (0, 4, 3)
       "show-scale", TRUE,
@@ -196,6 +198,10 @@ create_map_area (MapBuddyWindow *self)
       "notify::state",
       G_CALLBACK (on_state_changed),
       self);
+  g_signal_connect (self->view,
+      "notify::zoom-level",
+      G_CALLBACK (on_zoom_level_changed),
+      self);
 
   self->layer = champlain_layer_new ();
   champlain_view_add_layer (self->view, CHAMPLAIN_LAYER (self->layer));
@@ -409,10 +415,10 @@ create_toolbar (MapBuddyWindow *self)
   gtk_toolbar_insert (GTK_TOOLBAR (self->toolbar), GTK_TOOL_ITEM (entry), 2);
 }
 
-#define MAP_DRIVING "Driving"
-#define MAP_CYCLE "Cycling"
-#define MAP_TRANSIT "Public Transit"
-#define MAP_TERRAIN "Terrain"
+#define MAP_DRIVING _("Driving")
+#define MAP_CYCLE _("Cycling")
+#define MAP_TRANSIT _("Public Transit")
+#define MAP_TERRAIN _("Terrain")
 
 static GtkWidget *
 create_map_selector ()
@@ -422,13 +428,13 @@ create_map_selector ()
   selector = hildon_touch_selector_new_text();
 
   hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector),
-                                     _(MAP_DRIVING));
+                                     MAP_DRIVING);
   hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector),
-                                     _(MAP_CYCLE));
+                                     MAP_CYCLE);
   hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector),
-                                     _(MAP_TRANSIT));
+                                     MAP_TRANSIT);
   hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector),
-                                     _(MAP_TERRAIN));
+                                     MAP_TERRAIN);
 
   return selector;
 }
@@ -442,9 +448,9 @@ create_search_selector ()
   selector = hildon_touch_selector_new_text();
 
   hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector),
-                                     _(SEARCH_PLACES));
+                                     SEARCH_PLACES);
   hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector),
-                                     _(SEARCH_PRAIZED));
+                                     SEARCH_PRAIZED);
 
   return selector;
 }
@@ -483,7 +489,7 @@ create_menu (MapBuddyWindow *self)
       G_CALLBACK (on_copy_link), self);
 
   add_menu_item (main_menu, _("Donate"),
-      G_CALLBACK (open_url), "http://www.pierlux.com/mapbuddy/donate/");
+      G_CALLBACK (open_url), "http://www.pierlux.com/map-buddy/donate/");
 
   add_menu_item (main_menu, _("About"),
       G_CALLBACK (on_about), self);
@@ -526,9 +532,9 @@ on_about (GtkButton *button,
   dialog = gtk_dialog_new_with_buttons (_("About Map Buddy"),
       GTK_WINDOW (self),
       GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-      "License",
+      _("License"),
       GTK_RESPONSE_HELP,
-      "Web Site",
+      _("Web Site"),
       GTK_RESPONSE_NO,
       "Praized Media",
       GTK_RESPONSE_APPLY,
@@ -542,7 +548,7 @@ on_about (GtkButton *button,
   text = g_strdup_printf ("<big><b>%s %s</b></big>\n\n%s\n\n<small>%s</small>",
       _("Map Buddy"),
       VERSION,
-      "Copyright \xc2\xa9 2009 Pierre-Luc Beaudoin",
+      _("Copyright \xc2\xa9 2009 Pierre-Luc Beaudoin"),
       _("A map viewer for Maemo 5.\n\nThe maps are from the OpenStreetMap project and\nmapsforfree.com.\n\nPlace search is provided by geonames.org.\n\nBusinesses search is powered by Praized Media.\nUS data provided by Localeze.\nCanadian business listings distributed by YellowPages.ca™"));
 
   label = gtk_label_new (NULL);
@@ -567,10 +573,10 @@ on_about (GtkButton *button,
         open_url (NULL, "http://www.localeze.com");
         break;
       case GTK_RESPONSE_APPLY:
-        open_url (NULL, "http://www.praized.com");
+        open_url (NULL, "http://www.praizedmedia.com");
         break;
       case GTK_RESPONSE_NO:
-        open_url (NULL, "http://www.pierlux.com/mapbuddy/");
+        open_url (NULL, "http://www.pierlux.com/map-buddy/");
         break;
       case GTK_RESPONSE_HELP:
         dialog = gtk_dialog_new ();
@@ -693,15 +699,16 @@ on_layer (GtkButton *button,
     }
 
   selection = hildon_touch_selector_get_current_text (HILDON_TOUCH_SELECTOR (selector));
-  if (strcmp (selection, _(MAP_DRIVING)) == 0)
+  if (strcmp (selection, MAP_DRIVING) == 0)
     id = CHAMPLAIN_MAP_SOURCE_OSM_MAPNIK;
-  else if (strcmp (selection, _(MAP_CYCLE)) == 0)
+  else if (strcmp (selection, MAP_CYCLE) == 0)
     id = CHAMPLAIN_MAP_SOURCE_OSM_CYCLE_MAP;
-  else if (strcmp (selection, _(MAP_TRANSIT)) == 0)
+  else if (strcmp (selection, MAP_TRANSIT) == 0)
     id = CHAMPLAIN_MAP_SOURCE_OSM_TRANSPORT_MAP;
-  else if (strcmp (selection, _(MAP_TERRAIN)) == 0)
+  else if (strcmp (selection, MAP_TERRAIN) == 0)
     id = CHAMPLAIN_MAP_SOURCE_MFF_RELIEF;
 
+  g_print ("sel: %s, id: %s\n", selection, id);
   self->current_layer = id;
   factory = champlain_map_source_factory_dup_default ();
   source = champlain_map_source_factory_create (factory, id);
@@ -757,6 +764,49 @@ on_location_error (LocationGPSDControl *control,
   self->has_fix = FALSE;
 }
 
+static void
+update_me_marker_radius (MapBuddyWindow *self)
+{
+  gfloat radius = 0; /* in px */
+
+  /* This is necessary because Markers don't have access to the current zoom
+   * level */
+#if CHAMPLAIN_CHECK_VERSION (0, 4, 3)
+  if (self->has_fix)
+    {
+      ChamplainMapSource *source;
+      gint zoom;
+
+      g_object_get (self->view,
+          "map-source", &source,
+          "zoom-level", &zoom,
+          NULL);
+
+      radius = self->pos_error;
+      radius /= champlain_map_source_get_meters_per_pixel (source,
+          zoom, self->latitude, self->longitude);
+
+      if (radius > 600)
+        radius = 0; /* Drawing a too big circle jam the device */
+
+      g_object_unref (source);
+    }
+#endif
+
+  map_buddy_me_marker_set_radius (MAP_BUDDY_ME_MARKER (self->me_marker),
+                                  radius);
+
+}
+
+
+static void
+on_zoom_level_changed (GObject *gobject,
+                       GParamSpec *pspec,
+                       MapBuddyWindow *self)
+{
+  update_me_marker_radius (self);
+}
+
 
 static void
 on_location_changed (LocationGPSDevice *device,
@@ -775,14 +825,14 @@ on_location_changed (LocationGPSDevice *device,
       champlain_base_marker_set_position (CHAMPLAIN_BASE_MARKER (self->me_marker),
                                           device->fix->latitude,
                                           device->fix->longitude);
-      /* Temporary disable until tested, so far eph is always nan
-       * map_buddy_me_marker_set_radius (MAP_BUDDY_ME_MARKER (self->me_marker),
-                                      device->fix->eph / 100);
-       */
-      clutter_actor_show (CLUTTER_ACTOR (self->me_marker));
+
       self->has_fix = TRUE;
       self->latitude = device->fix->latitude;
       self->longitude = device->fix->longitude;
+      self->pos_error = device->fix->eph / 100.0;
+
+      update_me_marker_radius (self);
+      clutter_actor_show (CLUTTER_ACTOR (self->me_marker));
     }
   else
     {
@@ -988,7 +1038,7 @@ show_welcome_dialog (MapBuddyWindow *self)
   gtk_container_add (GTK_CONTAINER (content_area), label);
 
   check = hildon_check_button_new (HILDON_SIZE_AUTO);
-  gtk_button_set_label (GTK_BUTTON (check), "Don't remind me");
+  gtk_button_set_label (GTK_BUTTON (check), _("Don't remind me"));
   gtk_widget_show (check);
   gtk_container_add (GTK_CONTAINER (content_area), check);
 
@@ -1065,6 +1115,59 @@ open_url (GtkButton* button,
 }
 
 
+static void
+add_to_abook (GtkButton* button,
+              MapBuddyMerchant *merchant)
+{
+  OssoABookContact *contact;
+  GtkWidget *dialog;
+
+  contact = osso_abook_contact_new ();
+  if (merchant->name)
+    osso_abook_contact_set_value (E_CONTACT (contact), EVC_ORG, merchant->name);
+  if (merchant->url)
+    osso_abook_contact_set_value (E_CONTACT (contact), EVC_URL, merchant->url);
+  if (merchant->phone)
+    {
+
+      GRegex *reg = g_regex_new ("[\\(\\)\\ \\-]", 0, 0, NULL);
+      gchar *phone;
+
+      phone = g_regex_replace (reg, merchant->phone, -1, 0, "", 0, NULL);
+
+      osso_abook_contact_set_value (E_CONTACT (contact), EVC_TEL, phone);
+
+      g_free (phone);
+    }
+  if (merchant->address &&
+      merchant->city &&
+      merchant->province &&
+      merchant->postal_code &&
+      merchant->country)
+    {
+      EVCardAttribute *attr;
+
+      attr = e_vcard_attribute_new (NULL, EVC_ADR);
+
+      e_vcard_add_attribute_with_values (E_VCARD (contact), attr,
+          "", "", /* the first 2 elements are empty */
+          merchant->address,
+          merchant->city,
+          merchant->province,
+          merchant->postal_code,
+          merchant->country,
+          NULL);
+
+      /* attr ownership has been transfered to contact, don't free it */
+    }
+
+  dialog = osso_abook_temporary_contact_dialog_new_with_contact (NULL, contact);
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+
+  g_object_unref (contact);
+}
+
 static void
 place_call (GtkButton* button,
             const gchar* number)
@@ -1072,19 +1175,11 @@ place_call (GtkButton* button,
   DBusGConnection *connection;
   GError *error = NULL;
   DBusGProxy *proxy;
-  gchar stripped[12];
-  int i = 0;
-  int j = 0;
+  gchar *stripped;
+  GRegex *reg = g_regex_new ("[\\(\\)\\ \\-]", 0, 0, NULL);
+  gchar *phone;
 
-  for (i = 0; i < strlen (number); i++)
-    {
-      if (number[i] != '(' &&
-          number[i] != ')' &&
-          number[i] != ' ' &&
-          number[i] != '-')
-        stripped[j++] = number[i];
-    }
-  stripped[j] = '\0';
+  stripped = g_regex_replace (reg, number, -1, 0, "", 0, NULL);
 
   connection = dbus_g_bus_get (DBUS_BUS_SYSTEM,
                                &error);
@@ -1123,6 +1218,7 @@ place_call (GtkButton* button,
 
   g_object_unref (proxy);
   dbus_g_connection_unref (connection);
+  g_free (stripped);
 }
 
 void
@@ -1136,13 +1232,13 @@ map_buddy_window_show_details (MapBuddyWindow *self,
   GString *license;
 
   win = hildon_stackable_window_new ();
-  gtk_window_set_title (GTK_WINDOW (win), "Merchant detail");
+  gtk_window_set_title (GTK_WINDOW (win), _("Merchant detail"));
 
-  license = g_string_new ("Powered by Praized Media\n");
+  license = g_string_new (_("Powered by Praized Media\n"));
   if (g_strcmp0 (merchant->country_code, "CA") == 0)
-    g_string_append (license, "Canadian business listings distributed by YellowPages.ca™");
+    g_string_append (license, _("Canadian business listings distributed by YellowPages.ca\u2122"));
   else if (g_strcmp0 (merchant->country_code, "US") == 0)
-    g_string_append (license, "US data provided by Localeze");
+    g_string_append (license, _("US data provided by Localeze"));
 
   text = g_strdup_printf ("<big><b>%s</b></big>\n\n%s\n\n%s\n%s (%s)\n%s\n\n<span font=\"8\">%s</span>",
                         merchant->name,
@@ -1185,6 +1281,19 @@ map_buddy_window_show_details (MapBuddyWindow *self,
                           TRUE,
                           TRUE,
                           0);
+
+      button = hildon_button_new_with_text (HILDON_SIZE_FINGER_HEIGHT,
+                                            HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+                                            _("Add to Address Book"),
+                                            NULL);
+      gtk_widget_show (button);
+      g_signal_connect (G_OBJECT (button), "clicked",
+                        G_CALLBACK (add_to_abook), merchant);
+      gtk_box_pack_start (GTK_BOX (bbox),
+                          button,
+                          TRUE,
+                          TRUE,
+                          0);
     }
 
   if (merchant->url)
